/*
 * This confidential and proprietary software may be used only as
 * authorised by a licensing agreement from ARM Limited
 *    (C) COPYRIGHT 2013 ARM Limited
 *        ALL RIGHTS RESERVED
 * The entire notice above must be reproduced on all authorised
 * copies and copies may only be made to the extent permitted
 * by a licensing agreement from ARM Limited.
 */
 
float yuv_clip_f(float a, float amin, float amax)
{   
    if      (a < amin) return amin;
    else if (a > amax) return amax;
    else               return a;
} 
 
 
uchar do_chromakey_pixel(int bu,int bv,float blend,float similarity, uchar u[9], uchar v[9])
{   
    float diff = 0.0;
    int du, dv, i;
    
   #if 1
    
    for (i = 0; i < 9; ++i) {
        du = (int)u[i] - bu;
        dv = (int)v[i] - bv;
        diff += sqrt((du * du + dv * dv) / (255.0 * 255.0));
    }
    #endif
    
    diff /= 9.0;
    
    if (blend > 0.0001) {
        return (int)yuv_clip_f((diff - similarity) / blend, 0.0, 1.0) * 255.0;
    } else {
        return (diff > similarity) ? 255 : 0;
    }
}

void nv12_get_pixel_uv(int width,int height,
                    __global uchar *uv,
                    int x, int y, uchar *u, uchar *v)
{
   #if 0
    if (x < 0 || x >= width || y < 0 || y >= height)
        return;
#else
    if (x < 0) x = 0;
    if (x>=width) x=width-1;
    if (y<0) y = 0;
    if (y>=height) y=height-1;
#endif
    x >>= 1;
    y >>= 1;

    *u = uv[(width) * y + x*2];
    *v = uv[(width) * y + x*2+1];
}

void yv12_get_pixel_uv(int width,int height,
                    __global uchar *pu,__global uchar *pv,
                    int x, int y, uchar *u, uchar *v)
{   
#if 0
    if (x < 0 || x >= width || y < 0 || y >= height)
        return;
#else
    if (x < 0) x = 0;
    if (x>=width) x=width-1;
    if (y<0) y = 0;
    if (y>=height) y=height-1;
#endif
    
    x >>= 1;
    y >>= 1;
    
    *u = pu[(width>>1) * y + x];
    *v = pv[(width>>1) * y + x];    
}


 /**
 * \brief Mandelbrot kernel function.
 * \details Evaluates whether a set of four adjacent pixels are part of the Mandelbrot
 *          set. The output is an array where each value represents how many iterations
 *          were required to be able to decide whether the point in imaginary space
 *          is part of the Mandelbrot set or not. A value of MAX_ITER represents a point
 *          that is part of the Mandelbrot set. All other values represent points that are
 *          not part of the Mandelbrot set.
 *          A value is in the Mandelbrot set if the results of the calculations remain bounded.
 * \param[out] output Output data buffer. Must be width * height * sizeof(cl_uchar) in size.
 * \param[in] width Width of the data required.
 * \param[in] height Height of the data required.
 */
__kernel void yuv_chromakey_slice(__global uchar *Dst,
												 __global uchar* pu,	
												 __global uchar* pv,	
                         const int width,
                         const int height,int bu,int bv,float blend,float similarity)
{
		int xo, yo;
		uchar u[9] , v[9];

		#if 1
    /* [Kernel size] */
    /*
     * Each kernel calculates 4 output pixels in the same row (hence the '* 4').
     * x is in the range [0, width] .
     * y is in the range [0, height].
     */
    int x = get_global_id(0);
    int y = get_global_id(1);
    /* [Kernel size] */

		if (pu == pv) {
	    for (yo = 0; yo < 3; ++yo) {
	        for (xo = 0; xo < 3; ++xo) {
	        	nv12_get_pixel_uv(width,height, pu,x + xo - 1, y + yo - 1, &u[yo * 3 + xo], &v[yo * 3 + xo]);
	        }
	    }			
		} else {
	    for (yo = 0; yo < 3; ++yo) {
	        for (xo = 0; xo < 3; ++xo) {
	        	yv12_get_pixel_uv(width,height, pu,pv,x + xo - 1, y + yo - 1, &u[yo * 3 + xo], &v[yo * 3 + xo]);
	        }
	    }		
		}

    Dst[width*y + x] = do_chromakey_pixel( bu, bv, blend, similarity, u, v);
    #endif
}
