#extension GL_OES_EGL_image_external : require

varying highp vec2 v_TexCoordinate;

uniform samplerExternalOES inputImageTexture;
//uniform sampler2D inputImageTexture;
uniform lowp float brightness;
uniform lowp float contrast;
uniform lowp float saturation;
uniform lowp float exposure;

// Values from \Graphics Shaders: Theory and Practice\ by Bailey and Cunningham
const mediump vec3 luminanceWeighting = vec3(0.2125, 0.7154, 0.0721);

uniform mediump float hueAdjust;
const highp vec4 kRGBToYPrime = vec4 (0.299, 0.587, 0.114, 0.0);
const highp vec4 kRGBToI = vec4 (0.595716, -0.274453, -0.321263, 0.0);
const highp vec4 kRGBToQ = vec4 (0.211456, -0.522591, 0.31135, 0.0);

const highp vec4 kYIQToR = vec4 (1.0, 0.9563, 0.6210, 0.0);
const highp vec4 kYIQToG = vec4 (1.0, -0.2721, -0.6474, 0.0);
const highp vec4 kYIQToB = vec4 (1.0, -1.1070, 1.7046, 0.0);

void main() {
    lowp vec4 textureColor = texture2D(inputImageTexture, v_TexCoordinate);

//     gl_FragColor = vec4((textureColor.rgb + vec3(brightness)), textureColor.w);

     lowp vec4 brightnessTexColor = vec4((textureColor.rgb + vec3(brightness)), textureColor.w);
     lowp vec4 contrastTexColor = vec4(((brightnessTexColor.rgb - vec3(0.5)) * contrast + vec3(0.5)), brightnessTexColor.w);

	 //饱和度
//     lowp float luminance = dot(contrastTexColor.rgb, luminanceWeighting);
//     lowp vec3 greyScaleColor = vec3(luminance);
//     lowp vec4 color  = vec4(mix(greyScaleColor, contrastTexColor.rgb, saturation), contrastTexColor.w);
//	 //gl_FragColor = vec4(mix(greyScaleColor, contrastTexColor.rgb, saturation), contrastTexColor.w);

	 //自然饱和度
	 lowp float average = (contrastTexColor.r + contrastTexColor.g + contrastTexColor.b) / 3.0;
     lowp float mx = max(contrastTexColor.r, max(contrastTexColor.g, contrastTexColor.b));
     lowp float amt = (mx - average) * (-saturation * 3.0);
     lowp vec4 color = vec4(mix(contrastTexColor.rgb, vec3(mx), amt), contrastTexColor.w);


	if(hueAdjust != 0.0){
		// Convert to YIQ
		highp float YPrime = dot (color, kRGBToYPrime);
		highp float I = dot (color, kRGBToI);
		highp float Q = dot (color, kRGBToQ);

		// Calculate the hue and chroma
		highp float hue = atan (Q, I);
		highp float chroma = sqrt (I * I + Q * Q);

		// Make the user's adjustments
		hue += (-hueAdjust); //why negative rotation?

		// Convert back to YIQ
		Q = chroma * sin (hue);
		I = chroma * cos (hue);

		// Convert back to RGB
		highp vec4 yIQ = vec4 (YPrime, I, Q, 0.0);
		color.r = dot (yIQ, kYIQToR);
		color.g = dot (yIQ, kYIQToG);
		color.b = dot (yIQ, kYIQToB);
     }

	 //曝光
	 color = vec4(color.rgb * pow(2.0, exposure), color.w);// rgb * 2^效果值

     gl_FragColor = color;

}