#extension GL_OES_EGL_image_external : require
precision mediump float;

varying highp vec2 v_TexCoordinate;
varying highp vec4 postionVarying;
uniform samplerExternalOES texture;

vec4 cb_v4 = vec4( -0.100644, -0.338572,  0.439216, 0.501961 );
vec4 cr_v4 = vec4( 0.439216, -0.398942, -0.040274, 0.501961 );

float opacity;
uniform float contrast;
uniform float brightness;
uniform float gamma;
uniform float cb;
uniform float cr;
uniform float widthDiv;
uniform float heightDiv;

 vec2 chroma_key;
 vec2 pixel_size;
uniform float similarity;
uniform float smoothness;
uniform float spill;

uniform int uni_isStartHsv;
uniform int shapeFilter;
uniform int shapeFilterZoomXy;
uniform float shapeFilterWidth;
uniform float shapeFilterHeight;
uniform float shapeFilterTexWidth;
uniform float shapeFilterTexHeight;

////分割模式
uniform float modeDivXPars;
uniform float modeDivAngle;
uniform float modeDivTranslation;
uniform float modeDivThickness;
uniform int modeDivType;

vec4 CalcColor(vec4 rgba)
{   
    return vec4(pow(rgba.rgb, vec3(gamma, gamma, gamma)) * contrast + brightness, rgba.a);
}

float GetChromaDist(vec3 rgb)
{   
	float cb1 = dot(rgb.rgb, cb_v4.xyz) + cb_v4.w;
	float cr1 = dot(rgb.rgb, cr_v4.xyz) + cr_v4.w;
	return distance(chroma_key, vec2(cb1, cr1));
}

float GetNonlinearChannel(float u)
{
	return (u <= 0.0031308) ? (12.92 * u) : ((1.055 * pow(u, 1.0 / 2.4)) - 0.055);
}

vec3 GetNonlinearColor(vec3 rgb)
{
	return vec3(GetNonlinearChannel(rgb.r), GetNonlinearChannel(rgb.g), GetNonlinearChannel(rgb.b));
}

vec3 SampleTexture(vec2 uv)
{
    vec3 rgb = texture2D(texture, uv).rgb;
    return GetNonlinearColor(rgb);
}

float GetBoxFilteredChromaDist(vec3 rgb, vec2 texCoord)
{
    vec2 h_pixel_size = pixel_size / 2.0;
    vec2 point_0 = vec2(pixel_size.x, h_pixel_size.y);
    vec2 point_1 = vec2(h_pixel_size.x, -pixel_size.y);
    float distVal = GetChromaDist(SampleTexture(texCoord-point_0));
    distVal += GetChromaDist(SampleTexture(texCoord+point_0));
    distVal += GetChromaDist(SampleTexture(texCoord-point_1));
    distVal += GetChromaDist(SampleTexture(texCoord+point_1));
    distVal *= 2.0;
    distVal += GetChromaDist(GetNonlinearColor(rgb));
    return distVal / 9.0;
}

vec4 ProcessChromaKey(vec4 rgba)
{
    float chromaDist = GetBoxFilteredChromaDist(rgba.rgb, v_TexCoordinate);
    float baseMask = chromaDist - similarity;
    float fullMask = pow(clamp(baseMask / smoothness, 0.0, 1.0), 1.5);
    float spillVal = pow(clamp(baseMask / spill, 0.0, 1.0), 1.5);

	rgba.a *= opacity;
	rgba.a *= fullMask;

	float desat = dot(rgba.rgb, vec3(0.2126, 0.7152, 0.0722));
	//rgba.rgb = lerp(vec3(desat, desat, desat), rgba.rgb, spillVal);
	rgba.rgb = mix(vec3(desat, desat, desat), rgba.rgb, spillVal);

	return CalcColor(rgba);
}

void main(void)
{
	vec4 rgba = texture2D(texture, v_TexCoordinate);
	if(uni_isStartHsv == 1){
		opacity = 1.0;
		chroma_key=vec2(cb, cr);
		pixel_size = vec2(widthDiv, heightDiv);
		rgba.rgb = max(vec3(0.0, 0.0, 0.0), rgba.rgb / rgba.a);
		gl_FragColor = ProcessChromaKey(rgba);
		gl_FragColor.rgb *= gl_FragColor.a;
	}else{
		if(shapeFilter == 1){ //圆形
			float texScale = shapeFilterTexWidth/shapeFilterTexHeight;
			float vertexScale = shapeFilterWidth/shapeFilterHeight;
			float r = 0.5;
			if(texScale > vertexScale){ //
				if(shapeFilterZoomXy == 0){
					//old
					//r = ((shapeFilterWidth*shapeFilterTexHeight)/(shapeFilterTexWidth*shapeFilterHeight))/2.0;
					//if(pow((v_TexCoordinate.x-0.5), 2.0) + pow((v_TexCoordinate.y-0.5)*shapeFilterTexHeight/shapeFilterTexWidth, 2.0) > pow(r, 2.0)){
					//	rgba = vec4(0.0, 0.0, 0.0, 0.0);
					//}
					if((shapeFilterHeight*shapeFilterTexWidth)<(shapeFilterTexWidth*shapeFilterWidth)){
						r = ((shapeFilterWidth*shapeFilterTexHeight)/(shapeFilterTexWidth*shapeFilterHeight))/2.0;
						if(pow((v_TexCoordinate.x-0.5)*shapeFilterTexWidth/shapeFilterTexHeight, 2.0) + pow((v_TexCoordinate.y-0.5), 2.0) > pow(r, 2.0)){
							rgba = vec4(0.0, 0.0, 0.0, 0.0);
						}
					}else{
						r = ((shapeFilterWidth*shapeFilterTexHeight)/(shapeFilterTexWidth*shapeFilterHeight))/2.0;
						if(pow((v_TexCoordinate.x-0.5), 2.0) + pow((v_TexCoordinate.y-0.5)*shapeFilterTexHeight/shapeFilterTexWidth, 2.0) > pow(r, 2.0)){
							rgba = vec4(0.0, 0.0, 0.0, 0.0);
						}
					}
				}else{
					if((shapeFilterHeight*shapeFilterTexWidth)<(shapeFilterTexWidth*shapeFilterWidth)){
						r = ((shapeFilterWidth*shapeFilterTexHeight)/(shapeFilterTexWidth*shapeFilterHeight))/2.0;
						if(pow((v_TexCoordinate.x-0.5), 2.0) + pow((v_TexCoordinate.y-0.5)*shapeFilterTexWidth/shapeFilterTexHeight, 2.0) > pow(r, 2.0)){
							rgba = vec4(0.0, 0.0, 0.0, 0.0);
						}
					}else{
						r = ((shapeFilterWidth*shapeFilterTexHeight)/(shapeFilterTexWidth*shapeFilterHeight))/2.0;
						if(pow((v_TexCoordinate.x-0.5)*shapeFilterTexHeight/shapeFilterTexWidth, 2.0) + pow((v_TexCoordinate.y-0.5), 2.0) > pow(r, 2.0)){
							rgba = vec4(0.0, 0.0, 0.0, 0.0);
						}
					}
				}
			}else if(texScale < vertexScale){  //h 大 w=
				if(shapeFilterZoomXy == 0){
					if((shapeFilterHeight*shapeFilterTexWidth)<(shapeFilterTexWidth*shapeFilterWidth)){ //顶点高度比 tex 宽度大？
						r = ((shapeFilterHeight*shapeFilterTexWidth))/(shapeFilterTexHeight*shapeFilterWidth)/2.0;
						if(pow((v_TexCoordinate.x-0.5)*shapeFilterTexWidth/shapeFilterTexHeight, 2.0) + pow((v_TexCoordinate.y-0.5), 2.0) > pow(r, 2.0)){
							rgba = vec4(0.0, 0.0, 0.0, 0.0);
						}
					}else{
                		if(pow((v_TexCoordinate.x-0.5), 2.0) + pow((v_TexCoordinate.y-0.5)*shapeFilterTexHeight/shapeFilterTexWidth, 2.0) > pow(r, 2.0)){
							rgba = vec4(0.0, 0.0, 0.0, 0.0);
						}
					}
				}else{
					if((shapeFilterHeight*shapeFilterTexWidth)<(shapeFilterTexWidth*shapeFilterWidth)){ //顶点高度比 tex 宽度大？
						r = ((shapeFilterHeight*shapeFilterTexWidth))/(shapeFilterTexHeight*shapeFilterWidth)/2.0;
						if(pow((v_TexCoordinate.x-0.5), 2.0) + pow((v_TexCoordinate.y-0.5)*shapeFilterTexWidth/shapeFilterTexHeight, 2.0) > pow(r, 2.0)){
							rgba = vec4(0.0, 0.0, 0.0, 0.0);
						}
					}else{
                		if(pow((v_TexCoordinate.x-0.5)*shapeFilterTexHeight/shapeFilterTexWidth, 2.0) + pow((v_TexCoordinate.y-0.5), 2.0) > pow(r, 2.0)){
							rgba = vec4(0.0, 0.0, 0.0, 0.0);
						}
					}
				}
			}else{
				if(shapeFilterZoomXy == 0){
					if(shapeFilterTexHeight > shapeFilterTexWidth){
						if(pow((v_TexCoordinate.x-0.5), 2.0) + pow((v_TexCoordinate.y-0.5)*shapeFilterTexHeight/shapeFilterTexWidth, 2.0) > pow(r, 2.0)){
							rgba = vec4(0.0, 0.0, 0.0, 0.0);
						}
					}else{
						if(pow((v_TexCoordinate.x-0.5)*shapeFilterTexWidth/shapeFilterTexHeight, 2.0) + pow((v_TexCoordinate.y-0.5), 2.0) > pow(r, 2.0)){
							rgba = vec4(0.0, 0.0, 0.0, 0.0);
						}
					}
				}else{
					if(shapeFilterTexHeight > shapeFilterTexWidth){
						if(pow((v_TexCoordinate.x-0.5)*shapeFilterTexHeight/shapeFilterTexWidth, 2.0) + pow((v_TexCoordinate.y-0.5), 2.0) > pow(r, 2.0)){
							rgba = vec4(0.0, 0.0, 0.0, 0.0);
						}
					}else{
						if(pow((v_TexCoordinate.x-0.5), 2.0) + pow((v_TexCoordinate.y-0.5)*shapeFilterTexWidth/shapeFilterTexHeight, 2.0) > pow(r, 2.0)){
							rgba = vec4(0.0, 0.0, 0.0, 0.0);
						}
					}

				}
			}
		}else if(shapeFilter == 2){ //椭圆
			float texScale = shapeFilterTexWidth/shapeFilterTexHeight;
            float vertexScale = shapeFilterWidth/shapeFilterHeight;
            if(texScale > vertexScale){
            	if(shapeFilterZoomXy == 0){
            		if(pow((v_TexCoordinate.x-0.5)*((shapeFilterTexWidth*shapeFilterHeight)/(shapeFilterWidth*shapeFilterTexHeight)), 2.0)+ pow((v_TexCoordinate.y-0.5), 2.0) > 0.25){
						rgba = vec4(0.0, 0.0, 0.0, 0.0);
					}
            	}else{
            		if(pow((v_TexCoordinate.x-0.5), 2.0)+ pow((v_TexCoordinate.y-0.5)*((shapeFilterTexWidth*shapeFilterHeight)/(shapeFilterWidth*shapeFilterTexHeight)), 2.0) > 0.25){
						rgba = vec4(0.0, 0.0, 0.0, 0.0);
					}
            	}
            }else if(texScale < vertexScale){
            	if(shapeFilterZoomXy == 0){
					if(pow(v_TexCoordinate.x-0.5, 2.0) + pow((v_TexCoordinate.y-0.5)*((shapeFilterTexHeight*shapeFilterWidth)/(shapeFilterHeight*shapeFilterTexWidth)), 2.0) > 0.25){
						rgba = vec4(0.0, 0.0, 0.0, 0.0);
					}
				}else{
					if(pow((v_TexCoordinate.x-0.5)*((shapeFilterTexHeight*shapeFilterWidth)/(shapeFilterHeight*shapeFilterTexWidth)), 2.0) + pow((v_TexCoordinate.y-0.5), 2.0) > 0.25){
						rgba = vec4(0.0, 0.0, 0.0, 0.0);
					}
				}
			}else{
				if(pow(v_TexCoordinate.x-0.5, 2.0) + pow(v_TexCoordinate.y-0.5, 2.0) > 0.25){
					rgba = vec4(0.0, 0.0, 0.0, 0.0);
				}
			}
		}else if(shapeFilter == 111){ //圆形
			//(x-a)²+(y-b)²=r²
			if(shapeFilterZoomXy == 0){
				//if(pow((v_TexCoordinate.x-0.5)*shapeFilterHeight/shapeFilterWidth, 2.0) + pow((v_TexCoordinate.y-0.5), 2.0) > 0.25){
				//	rgba = vec4(0.0, 0.0, 0.0, 0.0);
				//}
				float texScale = shapeFilterTexWidth/shapeFilterTexHeight;
				float vertexScale = shapeFilterWidth/shapeFilterHeight;
				float r = 0.5;
				if(texScale > vertexScale){
					if(shapeFilterWidth < shapeFilterHeight){
						if(shapeFilterHeight < shapeFilterTexHeight){
							r = shapeFilterWidth/shapeFilterHeight/2.0;//?
						}else{
							r = shapeFilterWidth/shapeFilterTexWidth/2.0;
						}
					}else{

					}
				}else if(texScale < vertexScale){
					if(shapeFilterHeight < shapeFilterWidth){
						r = shapeFilterHeight/shapeFilterWidth/2.0;
					}else{

					}
				}else{
					if(shapeFilterWidth < shapeFilterHeight){

					}else{
						r = shapeFilterHeight/shapeFilterWidth/2.0;
					}
				}
				if(pow((v_TexCoordinate.x-0.5)*shapeFilterTexHeight/shapeFilterTexWidth, 2.0) + pow((v_TexCoordinate.y-0.5), 2.0) > pow(r, 2.0)){
					rgba = vec4(0.0, 0.0, 0.0, 0.0);
				  }
			}else{
				//if(pow((v_TexCoordinate.x-0.5), 2.0) + pow((v_TexCoordinate.y-0.5)*shapeFilterWidth/shapeFilterHeight, 2.0) > 0.25){
				//	rgba = vec4(0.0, 0.0, 0.0, 0.0);
				//}

				float texScale = shapeFilterTexWidth/shapeFilterTexHeight;
				float vertexScale = shapeFilterWidth/shapeFilterHeight;
				float r = 0.5;
				if(texScale > vertexScale){
					if(shapeFilterWidth < shapeFilterHeight){
						//r = shapeFilterWidth/shapeFilterHeight/2.0;
					}else{

					}
					if(pow((v_TexCoordinate.x-0.5), 2.0) + pow((v_TexCoordinate.y-0.5)*shapeFilterTexHeight/shapeFilterHeight, 2.0) > pow(r, 2.0)){
						rgba = vec4(0.0, 0.0, 0.0, 0.0);
					}

				}else if(texScale < vertexScale){

					//if(shapeFilterHeight < shapeFilterWidth){
						r = shapeFilterHeight/shapeFilterWidth/2.0;
					//}else{

					//}

					if(pow((v_TexCoordinate.x-0.5)*shapeFilterTexWidth/shapeFilterWidth, 2.0) + pow((v_TexCoordinate.y-0.5), 2.0) > pow(r, 2.0)){
						rgba = vec4(0.0, 0.0, 0.0, 0.0);
					}

				}else{
					float r = 0.5;
					if(shapeFilterWidth < shapeFilterHeight){
						r = shapeFilterWidth/shapeFilterHeight/2.0;
					}else{

					}
					if(pow((v_TexCoordinate.x-0.5), 2.0) + pow((v_TexCoordinate.y-0.5)*shapeFilterTexWidth/shapeFilterTexHeight, 2.0) > pow(r, 2.0)){
						rgba = vec4(0.0, 0.0, 0.0, 0.0);
					}
				}
			}
		}else{

		}

		//分割模式
		if(modeDivType == 0){
			gl_FragColor = rgba;
		}else{
			if(modeDivAngle == 90.0){ //垂直线
				float xValue = postionVarying.x + modeDivTranslation;
				if(xValue < modeDivThickness && xValue > -modeDivThickness){
					gl_FragColor = vec4(1.0, 1.0, 1.0, 1.0);
				}else{
					gl_FragColor = rgba;
				}
			}else{
				float yValue = postionVarying.y- modeDivXPars*(postionVarying.x + modeDivTranslation);
				if(yValue < modeDivThickness && yValue > -modeDivThickness){
					gl_FragColor = vec4(1.0, 1.0, 1.0, 1.0);
				}else{
					if(modeDivType == 1 && yValue < 0.0){
						rgba = vec4(0.0, 0.0, 0.0, 0.0);
					}else if(modeDivType == 2 && yValue > 0.0){
						rgba = vec4(0.0, 0.0, 0.0, 0.0);
					}else{
						gl_FragColor = rgba;
					}
				}
			}
		}
	}
}
