varying highp vec2 textureCoordinate;

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture1;

//高斯模糊相关 in {0.0, 0.01}
float blurSizeA = 0.00195;
//in {0.00, 0.99}
uniform float threshold;
//in {0.00, 1.00}
uniform float amount;
vec2 iResolution = vec2(1.0/720.0, 1.0/1280.0);

uniform int horizontal;

//高斯模糊 均值模糊
vec4 gaussianBlur(vec2 fragCoord)
{
    vec4 sum = vec4(0);

    // blur in x (horizontal)
	if(horizontal == 1){
		// take nine samples, with the distance blurSizeB between them
        sum += texture2D(inputImageTexture, vec2(fragCoord.x - 4.0 * iResolution.x, fragCoord.y)) * 0.05;
        sum += texture2D(inputImageTexture, vec2(fragCoord.x - 3.0 * iResolution.x, fragCoord.y)) * 0.09;
        sum += texture2D(inputImageTexture, vec2(fragCoord.x - 2.0 * iResolution.x, fragCoord.y)) * 0.12;
        sum += texture2D(inputImageTexture, vec2(fragCoord.x - 1.0 * iResolution.x, fragCoord.y)) * 0.15;
        sum += texture2D(inputImageTexture, vec2(fragCoord.x, fragCoord.y)) * 0.16;
        sum += texture2D(inputImageTexture, vec2(fragCoord.x + 1.0 * iResolution.x, fragCoord.y)) * 0.15;
        sum += texture2D(inputImageTexture, vec2(fragCoord.x + 2.0 * iResolution.x, fragCoord.y)) * 0.12;
        sum += texture2D(inputImageTexture, vec2(fragCoord.x + 3.0 * iResolution.x, fragCoord.y)) * 0.09;
        sum += texture2D(inputImageTexture, vec2(fragCoord.x + 4.0 * iResolution.x, fragCoord.y)) * 0.05;
	}else{
		sum += texture2D(inputImageTexture1, vec2(fragCoord.x, fragCoord.y - 4.0 * iResolution.y)) * 0.05;
		sum += texture2D(inputImageTexture1, vec2(fragCoord.x, fragCoord.y - 3.0 * iResolution.y)) * 0.09;
		sum += texture2D(inputImageTexture1, vec2(fragCoord.x, fragCoord.y - 2.0 * iResolution.y)) * 0.12;
		sum += texture2D(inputImageTexture1, vec2(fragCoord.x, fragCoord.y - 1.0 * iResolution.y)) * 0.15;
		sum += texture2D(inputImageTexture1, vec2(fragCoord.x, fragCoord.y)) * 0.16;
		sum += texture2D(inputImageTexture1, vec2(fragCoord.x, fragCoord.y + 1.0 * iResolution.y)) * 0.15;
		sum += texture2D(inputImageTexture1, vec2(fragCoord.x, fragCoord.y + 2.0 * iResolution.y)) * 0.12;
		sum += texture2D(inputImageTexture1, vec2(fragCoord.x, fragCoord.y + 3.0 * iResolution.y)) * 0.09;
		sum += texture2D(inputImageTexture1, vec2(fragCoord.x, fragCoord.y + 4.0 * iResolution.y)) * 0.05;
	}

	//sum += texture2D(inputImageTexture, vec2(uv.x, uv.y - 4.0 * blurSizeA)) * 0.05;
    //sum += texture2D(inputImageTexture, vec2(uv.x, uv.y - 3.0 * blurSizeA)) * 0.09;
    //sum += texture2D(inputImageTexture, vec2(uv.x, uv.y - 2.0 * blurSizeA)) * 0.12;
    //sum += texture2D(inputImageTexture, vec2(uv.x, uv.y - 1.0 * blurSizeA)) * 0.15;
    //sum += texture2D(inputImageTexture, vec2(uv.x, uv.y)) * 0.16;
    //sum += texture2D(inputImageTexture, vec2(uv.x, uv.y + 1.0 * blurSizeA)) * 0.15;
    //sum += texture2D(inputImageTexture, vec2(uv.x, uv.y + 2.0 * blurSizeA)) * 0.12;
    //sum += texture2D(inputImageTexture, vec2(uv.x, uv.y + 3.0 * blurSizeA)) * 0.09;
    //sum += texture2D(inputImageTexture, vec2(uv.x, uv.y + 4.0 * blurSizeA)) * 0.05;

    return vec4(vec3(sum), 1.);
}

vec4 mask(vec4 blur, vec2 fragCoord)
{
    vec3 col = texture2D(inputImageTexture, fragCoord).rgb;
    vec3 blurCol = blur.rgb;

    vec3 v =  col - blurCol;
    bvec3 b = greaterThan(abs(v), vec3(threshold));
    return vec4(vec3(b), 1.);
}

vec4 usim(vec4 blur, vec4 mask, vec2 fragCoord)
{
    vec3 col = texture2D(inputImageTexture, fragCoord).rgb;
    vec3 blurCol = blur.rgb;
    vec3 maskCol = mask.rgb;

    vec3 v =  col - blurCol;

    v = col + (amount * v);

    col = clamp(mix(col, v, maskCol), 0., 1.);

    return vec4(col, 1.);
}


void main(){
	if(horizontal == 1){
		vec4 blurValue = gaussianBlur(textureCoordinate);
    	gl_FragColor = blurValue;
	}else{
		vec4 blurValue = gaussianBlur(textureCoordinate);
    	vec4 maskValue = mask(blurValue, textureCoordinate);
    	vec4 usimValue = usim(blurValue, maskValue, textureCoordinate);
    	gl_FragColor = usimValue;
	}
    //gl_FragColor = texture2D(inputImageTexture, textureCoordinate);
}