precision highp float;

varying vec2 textureCoordinate;

uniform sampler2D inputImageTexture;
uniform float uWidth;
uniform float uHeight;


const vec4 COEF_Y = vec4( 0.257,  0.504,  0.098, 0.0625);
const vec4 COEF_U = vec4(-0.148, -0.291,  0.439, 0.5);
const vec4 COEF_V = vec4( 0.439, -0.368, -0.071, 0.5);

//数字BT709  Y
//const vec4 COEF_Y = vec4( 0.183,  0.614,  0.062, 0.0625);
//const vec4 COEF_U = vec4(-0.148, -0.291,  0.439, 0.5);
//const vec4 COEF_V = vec4( 0.439, -0.368, -0.071, 0.5);

float cY(float x, float y)
{
    vec4 c = texture2D(inputImageTexture, vec2(x, y));
    //return clamp(c.r * 0.257 + c.g * 0.504 + c.b * 0.098 + 0.0625, 0.0, 1.0);
    return clamp(dot(c.rgb, COEF_Y.rgb) + COEF_Y.a, 0.0, 1.0);
}

vec4 cC(float x, float y, float dx, float dy)
{
    vec4 c0 = texture2D(inputImageTexture, vec2(x, y));
    vec4 c1 = texture2D(inputImageTexture, vec2(x + dx, y));
    vec4 c2 = texture2D(inputImageTexture, vec2(x, y + dy));
    vec4 c3 = texture2D(inputImageTexture, vec2(x + dx, y + dy));
    return (c0 + c1 + c2 + c3) / 4.0;
}

float cU(float x, float y, float dx, float dy)
{
    vec4 c = cC(x, y, dx, dy);
    //return clamp(-0.148 * c.r - 0.291 * c.g + 0.439 * c.b + 0.5000, 0.0, 1.0);
    return clamp(dot(c.rgb, COEF_U.rgb) + COEF_U.a, 0.0, 1.0);
}

float cV(float x, float y, float dx, float dy)
{
    vec4 c = cC(x, y, dx, dy);
    //return clamp(0.439 * c.r - 0.368 * c.g - 0.071 * c.b + 0.5000, 0.0, 1.0);
    return clamp(dot(c.rgb, COEF_V.rgb) + COEF_V.a, 0.0, 1.0);
}

vec4 calculateY(float x, float y, float dx, float dy)
{
    vec4 c = vec4(0.0);
    c[0] = cY(x, y);
    c[1] = cY(x + 1.0 * dx, y);
    c[2] = cY(x + 2.0 * dx, y);
    c[3] = cY(x + 3.0 * dx, y);
    return c;
}

vec4 calculateU(float x, float y, float dx, float dy)
{
    vec4 c = vec4(0.0);
    c[0] = cU(x, y, dx, dy);
    c[1] = cU(x + 2.0 * dx, y, dx, dy);
    c[2] = cU(x + 4.0 * dx, y, dx, dy);
    c[3] = cU(x + 6.0 * dx, y, dx, dy);
    return c;
}

vec4 calculateV(float x, float y, float dx, float dy)
{
    vec4 c = vec4(0.0);
    c[0] = cV(x, y, dx, dy);
    c[1] = cV(x + 2.0 * dx, y, dx, dy);
    c[2] = cV(x + 4.0 * dx, y, dx, dy);
    c[3] = cV(x + 6.0 * dx, y, dx, dy);
    return c;
}

void aaamain()
{
    if (textureCoordinate.y > 0.375) {
       //gl_FragColor=vec4(0,0,0,0);
    }else{
      	float y4xSum = uWidth * uHeight / 4.0;
        float u4xSum = uWidth * uHeight / 16.0;
        float v4xSum = uWidth * uHeight / 16.0;
        float yuv4xOffset = floor(textureCoordinate.y * uHeight) * uWidth + floor(textureCoordinate.x * uWidth);

        if (yuv4xOffset < y4xSum) {
            float yOffset = yuv4xOffset * 4.0 + 1.0;
            float yX = floor(mod(yOffset, uWidth));
            float yY = floor(yOffset / uWidth);
            yY = uHeight - yY;
            gl_FragColor = calculateY(yX / uWidth, yY / uHeight, 1.0 / uWidth, 1.0 / uHeight);
        } else if (yuv4xOffset < (y4xSum + u4xSum)) {
            float uOffset = (yuv4xOffset - y4xSum) * 4.0 + 1.0;
            float uX = floor(mod(uOffset, uWidth / 2.0)) * 2.0;
            float uY = floor(uOffset / (uWidth / 2.0)) * 2.0;
            uY = uHeight - uY - 1.0;
            gl_FragColor = calculateU(uX / uWidth, uY / uHeight, 1.0 / uWidth, 1.0 / uHeight);
        } else if (yuv4xOffset < (y4xSum + u4xSum + v4xSum)) {
            float vOffset = (yuv4xOffset - y4xSum - u4xSum) * 4.0 + 1.0;
            float vX = floor(mod(vOffset, uWidth / 2.0)) * 2.0;
            float vY = floor(vOffset / (uWidth / 2.0)) * 2.0;
            vY = uHeight - vY - 1.0;
            gl_FragColor = calculateV(vX / uWidth, vY / uHeight, 1.0 / uWidth, 1.0 / uHeight);
        }
    }
}

vec2 cPos(float t,float shiftx,float gy){
    vec2 pos=vec2(floor(uWidth*textureCoordinate.x),floor(uHeight*gy));
    return vec2(mod(pos.x*shiftx,uWidth),uHeight-(pos.y*shiftx+floor(pos.x*shiftx/uWidth))*t);
}

vec4 calculateY1(){
    vec2 pos=cPos(1.,4.,textureCoordinate.y);
    vec4 oColor=vec4(0);
    float textureYPos=pos.y/uHeight;
    oColor[0]=cY(pos.x/uWidth,textureYPos);
    oColor[1]=cY((pos.x+1.)/uWidth,textureYPos);
    oColor[2]=cY((pos.x+2.)/uWidth,textureYPos);
    oColor[3]=cY((pos.x+3.)/uWidth,textureYPos);
    return oColor;
}

vec4 calculateVU(float dx,float dy){
    vec2 pos=cPos(2.,4.,textureCoordinate.y-0.2500);
    vec4 oColor=vec4(0);
    float textureYPos=pos.y/uHeight;
    oColor[0]= cV(pos.x/uWidth,textureYPos,dx,dy);
    oColor[1]= cU(pos.x/uWidth,textureYPos,dx,dy);
    oColor[2]= cV((pos.x+2.)/uWidth,textureYPos,dx,dy);
    oColor[3]= cU((pos.x+2.)/uWidth,textureYPos,dx,dy);
    return oColor;
}

void main()
{
	if(textureCoordinate.y < 0.250){
		gl_FragColor=calculateY1();
	}else if(textureCoordinate.y < 0.375){
		gl_FragColor=calculateVU(1.0/uWidth, 1.0/uHeight);
	}else{

	}
}



