precision highp float;

varying vec2 textureCoordinate;

uniform sampler2D inputImageTexture;
uniform float uWidth;
uniform float uHeight;

float u_Offset;  //偏移量 1.0/width
//Y =  0.299R + 0.587G + 0.114B
//U = -0.147R - 0.289G + 0.436B
//V =  0.615R - 0.515G - 0.100B
//const vec3 COEF_Y = vec3( 0.299,  0.587,  0.114);
//const vec3 COEF_U = vec3(-0.147, -0.289,  0.436);
//const vec3 COEF_V = vec3( 0.615, -0.515, -0.100);

//BT.601 full range
//const vec3 COEF_Y = vec3( 0.299000,  0.587000,  0.114000);
//const vec3 COEF_U = vec3(-0.168736, -0.331264,  0.500000);
//const vec3 COEF_V = vec3( 0.500000, -0.418688, -0.081312);

//BT.601 limit range (Y + 0.0625, UV + 0.5)
const vec3 COEF_Y = vec3( 0.257,  0.504,  0.098);
const vec3 COEF_U = vec3(-0.148, -0.291,  0.439);
const vec3 COEF_V = vec3( 0.439, -0.368, -0.071);

//BT.2020 full range
//const vec3 COEF_Y = vec3( 0.262700,  0.678000,  0.059300);
//const vec3 COEF_U = vec3(-0.139630, -0.360370,  0.500000);
//const vec3 COEF_V = vec3( 0.500000, -0.459786, -0.040214);


//BT.709 limit range (Y + 0.0625, UV + 0.5)
//const vec3 COEF_Y = vec3( 0.182586,  0.628254,  0.063423);
//const vec3 COEF_U = vec3(-0.098397, -0.338572,  0.439216);
//const vec3 COEF_V = vec3( 0.429412, -0.398942, -0.040274);

//模拟BT709
//const vec3 COEF_Y = vec3( 0.212600,  0.715200,  0.072200);
//const vec3 COEF_U = vec3(-0.114572, -0.385428,  0.500000);
//const vec3 COEF_V = vec3( 0.500, -0.454153, -0.045847);

const float UV_DIVIDE_LINE = 2.0 / 3.0;
const float V_DIVIDE_LINE = 5.0 / 6.0;

void main()
{
    u_Offset = 1.0/uWidth;
    vec2 texelOffset = vec2(u_Offset, 0.0);
    if(textureCoordinate.y <= UV_DIVIDE_LINE) {
        //在纹理坐标 y < (2/3) 范围，需要完成一次对整个纹理的采样，
        //一次采样（加三次偏移采样）4 个 RGBA 像素（R,G,B,A）生成 1 个（Y0,Y1,Y2,Y3），整个范围采样结束时填充好 width*height 大小的缓冲区；

        vec2 texCoord = vec2(textureCoordinate.x, textureCoordinate.y * 3.0 / 2.0);
        vec4 color0 = texture2D(inputImageTexture, texCoord);
        vec4 color1 = texture2D(inputImageTexture, texCoord + texelOffset);
        vec4 color2 = texture2D(inputImageTexture, texCoord + texelOffset * 2.0);
        vec4 color3 = texture2D(inputImageTexture, texCoord + texelOffset * 3.0);

        float y0 = dot(color0.rgb, COEF_Y) + 0.0625;
        float y1 = dot(color1.rgb, COEF_Y) + 0.0625;
        float y2 = dot(color2.rgb, COEF_Y) + 0.0625;
        float y3 = dot(color3.rgb, COEF_Y) + 0.0625;
        gl_FragColor = vec4(y0, y1, y2, y3);
    }
    //i420
    else if(textureCoordinate.y <= V_DIVIDE_LINE){

            //当纹理坐标 y > (2/3) 且 y < (5/6) 范围，一次采样（加三次偏移采样）8 个 RGBA 像素（R,G,B,A）生成（U0,U1,U2,U3），
            //又因为 U plane 缓冲区的宽高均为原图的 1/2 ，U plane 在垂直方向和水平方向的采样都是隔行进行，整个范围采样结束时填充好 width*height/4 大小的缓冲区。

            float offsetY = 1.0 / 3.0 / uHeight;
            vec2 texCoord;
            if(textureCoordinate.x <= 0.5) {
                texCoord = vec2(textureCoordinate.x * 2.0, (textureCoordinate.y - UV_DIVIDE_LINE) * 2.0 * 3.0);
            }
            else {
                texCoord = vec2((textureCoordinate.x - 0.5) * 2.0, ((textureCoordinate.y - UV_DIVIDE_LINE) * 2.0 + offsetY) * 3.0);
            }

            vec4 color0 = texture2D(inputImageTexture, texCoord);
            vec4 color1 = texture2D(inputImageTexture, texCoord + texelOffset * 2.0);
            vec4 color2 = texture2D(inputImageTexture, texCoord + texelOffset * 4.0);
            vec4 color3 = texture2D(inputImageTexture, texCoord + texelOffset * 6.0);

            float u0 = dot(color0.rgb, COEF_U) + 0.5;
            float u1 = dot(color1.rgb, COEF_U) + 0.5;
            float u2 = dot(color2.rgb, COEF_U) + 0.5;
            float u3 = dot(color3.rgb, COEF_U) + 0.5;
            gl_FragColor = vec4(u0, u1, u2, u3);
        }
        else {
            //当纹理坐标 y > (5/6) 范围，一次采样（加三次偏移采样）8 个 RGBA 像素（R,G,B,A）生成（V0,V1,V2,V3），
            //同理，因为 V plane 缓冲区的宽高均为原图的 1/2 ，垂直方向和水平方向都是隔行采样，整个范围采样结束时填充好 width*height/4 大小的缓冲区。

            float offsetY = 1.0 / 3.0 / uHeight;
            vec2 texCoord;
            if(textureCoordinate.x <= 0.5) {
                texCoord = vec2(textureCoordinate.x * 2.0, (textureCoordinate.y - V_DIVIDE_LINE) * 2.0 * 3.0);
            }
            else {
                texCoord = vec2((textureCoordinate.x - 0.5) * 2.0, ((textureCoordinate.y - V_DIVIDE_LINE) * 2.0 + offsetY) * 3.0);
            }

            vec4 color0 = texture2D(inputImageTexture, texCoord);
            vec4 color1 = texture2D(inputImageTexture, texCoord + texelOffset * 2.0);
            vec4 color2 = texture2D(inputImageTexture, texCoord + texelOffset * 4.0);
            vec4 color3 = texture2D(inputImageTexture, texCoord + texelOffset * 6.0);

            float v0 = dot(color0.rgb, COEF_V) + 0.5;
            float v1 = dot(color1.rgb, COEF_V) + 0.5;
            float v2 = dot(color2.rgb, COEF_V) + 0.5;
            float v3 = dot(color3.rgb, COEF_V) + 0.5;
            gl_FragColor = vec4(v0, v1, v2, v3);
        }
//nv21
//    else {
//        //当纹理坐标 y > (2/3) 范围，一次采样（加三次偏移采样）4 个 RGBA 像素（R,G,B,A）生成 1 个（V0,U0,V0,U1），
//        //又因为 VU plane 缓冲区的高度为 height/2 ，VU plane 在垂直方向的采样是隔行进行，整个范围采样结束时填充好 width*height/2 大小的缓冲区。
//        vec2 texCoord = vec2(textureCoordinate.x, (textureCoordinate.y - UV_DIVIDE_LINE) * 3.0);
//        vec4 color0 = texture2D(inputImageTexture, texCoord);
//        vec4 color1 = texture2D(inputImageTexture, texCoord + texelOffset);
//        vec4 color2 = texture2D(inputImageTexture, texCoord + texelOffset * 2.0);
//        vec4 color3 = texture2D(inputImageTexture, texCoord + texelOffset * 3.0);
//
//        float v0 = dot(color0.rgb, COEF_V) + 0.501961;
//        float u0 = dot(color1.rgb, COEF_U) + 0.501961;
//        float v1 = dot(color2.rgb, COEF_V) + 0.501961;
//        float u1 = dot(color3.rgb, COEF_U) + 0.501961;
//        gl_FragColor = vec4(v0, u0, v1, u1);
//    }
}


